/*** EDB-GSB : Konvertierung von EDBS nach GSB ***/
/*   Version 1.00                                */
/*   letzte nderungen vom 06.03.98              */

/***** INCLUDES *****/
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

/***** STRUKTUREN *****/
struct punkt {
	int  nb,			/* Nummerierungsbezirk			*/
		 pa,			/* Punktart						*/
		 va,			/* Vermarkungsart				*/
		 ls,			/* Lagestatus					*/
		 ps,			/* Punktstatus					*/
		 yNk, xNk;		/* YX-Nachkommastellen			*/
	long nr,			/* Punktnummer					*/
		 yVk, xVk;		/* Y-Vorkommastellen			*/
}p;

/***** FUNKTIONEN *****/
void fehler(int err, ...);						/* Gibt Fehlermeldung aus	*/
char *upstr(char *s);							/* ndert String auf Grobst*/
char *strnncpy(char *dest, char *src, int a, int e);
												/* Kopiert Teilstring von
												   src nach dest			*/

/***** MAIN *****/
int main(int argc, char *argv[]) {
	/* Variablen */
	FILE *inFile,		/* Zeiger auf EDB-Datei	*/
		 *outFile;		/* Zeiger auf GSB-Datei */
	char *inName,		/* Dateiname  EDB-Datei */
		 *outName,		/* Dateiname  GSB-DAtei	*/
		 sbuf[500],		/* Puffer				*/
		 sWert[20];		/* Puffer fr Werte		*/
	int  s,				/* Punktsatz			*/
		 i;

	/* Meldung */
	printf("EDB-GSB Umsetzung von EDBS nach GSB\n\n");

	/* Argumente bernehmen */
	if(argc<3) fehler(1);
	strcpy(inName, argv[1]);
	if(!strchr(inName, '.'))
		strcat(inName, ".EDB");
	strcpy(outName, argv[2]);
	if(!strchr(outName, '.'))
		strcat(outName, ".GSB");

	/* Dateien ffnen */
	if((inFile  = fopen(inName , "r"))==NULL) fehler(2, inName);
	if((outFile = fopen(outName, "w"))==NULL) fehler(3, outName);

	/* Anzeigen der eingestellten Parameter */
	printf("Eingabedatei: %s\n", inName);
	printf("Ausgabedatei: %s\n", outName);

	/* Umsetzen */

	while(!feof(inFile)) {
		fgets(sbuf, 500, inFile);
		if(feof(inFile)) break;
		sscanf(strnncpy(sWert, sbuf, 17, 21), "%i", &s);
		if(s) {
			sscanf(strnncpy(sWert, sbuf,  48,  51), "%i" ,  &p.nb);
			sscanf(strnncpy(sWert, sbuf,  52,  52), "%i" ,  &p.pa);
			sscanf(strnncpy(sWert, sbuf,  53,  57), "%li",  &p.nr);
			sscanf(strnncpy(sWert, sbuf,  59,  59), "%i" ,  &p.ps);
			sscanf(strnncpy(sWert, sbuf,  71,  73), "%i" ,  &p.va);
			sscanf(strnncpy(sWert, sbuf, 117, 119), "%i" ,  &p.ls);
			sscanf(strnncpy(sWert, sbuf, 120, 126), "%li",  &p.yVk);
			sscanf(strnncpy(sWert, sbuf, 127, 129), "%i" ,  &p.yNk);
			sscanf(strnncpy(sWert, sbuf, 130, 136), "%li",  &p.xVk);
			sscanf(strnncpy(sWert, sbuf, 137, 139), "%i" ,  &p.xNk);

			fprintf(outFile, "%4i.%05li  %1i %03i          %03i  %1i  %07li.%03i %07li.%03i\n", p.nb, p.nr, p.pa, p.va, p.ls, p.ps, p.yVk, p.yNk, p.xVk, p.xNk);
		}
	}
	printf("\n");
	return 0;
}
void fehler(int err, ...) {
	va_list ap;
	va_start(ap, err);
	switch(err) {
		case  1: printf("EDB-GSB Umsetzung von EDBS nach GSB         \n"
						"Version 1.00                                \n"
						"(c) Jrg Schrder                           \n"
						"Aufruf: EDB-GSB <quelle> <ziel>             \n"
						"        <quelle>        EDB-Datei           \n"
						"        <ziel>          GSB-Datei           \n");
				 break;
		case  2: printf("Kann Datei %s nicht zum Lesen ffnen!\n", va_arg(ap, int));
				 break;
		case  3: printf("Kann Datei %s nicht zum Schreiben ffnen!\n", va_arg(ap, int));
				 break;
		default: printf("Error:%i Undef. Fehler!\n",err);
	}
	va_end(ap);
	exit(err);
	return;
}
char *upstr(char *s) {
	int i = 0;
	while (s[i]) {
		s[i] = toupper(s[i]);
		i++;
	}
	return s;
}
char *strnncpy(char *dest, char *src, int a, int e) {
	int i, j;
	for(i=a, j=0; i<e+1; i++, j++)
		dest[j] = src[i];
	dest[j]=0;
	for(i=0; i<j-1; i++)
		if(dest[i]=='0') dest[i]=' ';
		else break;
	return dest;
}