/*** GSB-EDB : Konvertierung von GSB nach EDBS ***/
/*   Version 1.02                                */
/*   letzte nderungen vom 26.06.98              */

/***** INCLUDES *****/
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>

/***** STRUKTUREN *****/
struct punkt {
	int  nb,			/* Numbez						*/
		 ny,			/* NumBez 1						*/
		 nx,            /* Numbez 2         			*/
		 pa,			/* Punktart						*/
		 va,			/* Vermarkungsart				*/
		 ls,			/* Lagestatus					*/
		 ps,			/* Punktstatus					*/
		 lga,			/* Art der Lagegenauigkeit		*/
		 jDb,			/* Jahr der Berechnung			*/
		 yNk, xNk,		/* YX-Nachkommastellen			*/
		 zSt,			/* zustndige Stelle
						   Index auf amt[]				*/
		 gSl;			/* Gemarkung
						   Index auf gem[]				*/
	long nr,			/* Punktnummer					*/
		 yVk, xVk,      /* YX-Vorkommastellen   		*/
		 aNr;			/* Antragsnummer				*/
}p;                     /* Punktsatz					*/
struct sl {
	char  name[9];		/* Name							*/
	long nr;			/* Nummer						*/
	int lga;			/* Lagegenauigkeit FP-Feld		*/
}amt[20],				/* Amt							*/
 gem[200];				/* Gemarkung					*/
struct option {
	char name[3];		/* Name							*/
	int  i;				/* Index auf argv				*/
}o[5]={{"-I", 0},
	   {"-A", 0},
	   {"-G", 0},
	   {"-J", 0},
	   {"-L", 0}};		/* Optionen						*/

/***** FUNKTIONEN *****/
void fehler(int err, ...);						/* Gibt Fehlermeldung aus   */
void nulltoBlank(char *buf, int a , int e);		/* setzt '0' zu ' '			*/
char *upstr(char *s);							/* ndert String auf Grobst*/
int  ssuchSl(struct sl *s, int se, char *st);	/* Sucht in sl nach string	*/
int  lsuchSl(struct sl *s, int se, long nr);	/* Sucht in sl nach long	*/
int  testLga(int lga);							/* Testet den Lagestatus	*/
int  testANr(char *s);

/***** MAIN *****/
int main(int argc, char *argv[]) {
	/* Variablen */
	struct date d;		/* Datum				*/
	FILE *infFile,		/* Zeiger auf INFdatei	*/
		 *inFile, 		/* Zeiger auf GSBdatei	*/
		 *outFile;		/* Zeiger auf EDBdatei	*/
	char *infName,		/* Dateiname  INFdatei	*/
		 inName[13],	/* Dateiname  GSBdatei	*/
		 outName[13],	/* Dateiname  EDBdatei	*/
		 sbuf[255];		/* Puffer				*/
	long lbuf;			/* Puffer				*/
	int  ibuf,			/* Puffer				*/
		 s=0,			/* Datensatznummer		*/
		 infSec=0,		/* Sektor in INFdatei
						   1 = Amt
						   2 = Gemeinde			*/
		 amtA=0,		/* Anzahl Amtschlssel	*/
		 gemA=0,		/* Anzahl Gemschlssel	*/
		 i, j, f;

	/* Meldung */
	printf("GSB-EDB Umsetzung von GSB nach EDBS\n\n");

	/* Argumente testen */
	if(argc<3) fehler(1);
	for(i=1; i<3; i++)
		if(argv[i][0]=='-') fehler(1);
	for(i=3; i<argc; i++)
		if(argv[i][0]!='-') fehler(10, i, argv[i]);

	for(i=3; i<argc; i++) {
		upstr(argv[i]);
		for(j=0; j<5; j++)
			if(!strncmp(argv[i], o[j].name, 2)) o[j].i = i;
		strnset(argv[i], ' ', 3);
	}

	/* Projektname bernehmen */
	strcpy(inName , argv[1]);
	strcat(inName , ".GSB");
	strcpy(outName, argv[1]);
	strcat(outName, ".EDB");

	/* Antragsnummer bernehmen */
	if(!testANr(argv[2])) fehler(9, argv[2]);
	sscanf(argv[2], "%li", &p.aNr);


	/* Datum ermitteln */
	getdate(&d);
	p.jDb = d.da_year-d.da_year/1000*1000;

	/* INF-Dateinamen bernehmen */
	if(o[0].i)
		sscanf(argv[o[0].i], "%255s", infName);
	else
		if((infName = getenv("GSB-EDB"))==NULL)
			fehler(4);

	/* Dateien ffnen */
	if((inFile =fopen(inName , "r"))==NULL) fehler(2, inName);
	if((outFile=fopen(outName, "w"))==NULL) fehler(3, outName);
	if((infFile=fopen(infName, "r"))==NULL) fehler(2, infName);

	/* Inf-Datei lesen */
	while(!feof(infFile)) {
		fgets(sbuf, 255, infFile);
		if(feof(infFile)) break;
		upstr(sbuf);
		if     (sbuf[0]=='[') {
			if     (!strncmp(sbuf, "[AMT]", 5))
				infSec = 1;
			else if(!strncmp(sbuf, "[GEMARKUNG]", 11))
				infSec = 2;
			else
				fehler(5, s+1, sbuf);
		}
		else if(isalpha(sbuf[0])) {
			if     (infSec == 1) {
				sscanf(sbuf, "%8s = %li", &amt[amtA].name, &amt[amtA].nr);
				amtA++;
			}
			else if(infSec == 2) {
				sscanf(sbuf, "%8s = %li %i", &gem[gemA].name, &gem[gemA].nr, &gem[gemA].lga);
				if(!testLga(gem[gemA].lga)) fehler(8, s+1, sbuf);
				gemA++;
			}
			else
				fehler(5, s+1, sbuf);
		}
		s++;
	}
	p.zSt=0; p.gSl=0;

	/* Argumente auswerten */
	if(o[1].i) {		/* Amt			*/
		if     (isdigit(argv[o[1].i][3])) {
			sscanf(argv[o[1].i], "%li", &lbuf);
			p.zSt = lsuchSl(amt, amtA, lbuf);
		}
		else if(isalpha(argv[o[1].i][3])) {
			sscanf(argv[o[1].i], "%s", sbuf);
			p.zSt = ssuchSl(amt, amtA, sbuf);
		}
		else {
			fehler(7, argv[o[1].i], o[1].name);
		}
		if(p.zSt==-1)  fehler(6, "s Amt", argv[o[1].i]);
	}
	if(o[2].i) {		/* Gemarkung	*/
		if     (isdigit(argv[o[2].i][3])) {
			sscanf(argv[o[2].i], "%li", &lbuf);
			p.gSl = lsuchSl(gem, gemA, lbuf);
		}
		else if(isalpha(argv[o[2].i][3])) {
			sscanf(argv[o[2].i], "%s", sbuf);
			p.gSl = ssuchSl(gem, gemA, sbuf);
		}
		else
			fehler(7, argv[o[2].i], o[2].name);
		if(p.gSl==-1) fehler(6, " Gemarkung", argv[o[2].i]);
	}
	if(o[3].i) {		/* Jahr der Berechnung	*/
		if     (isdigit(argv[o[3].i][3])) {
			sscanf(argv[o[3].i], "%i", &p.jDb);
			p.jDb = p.jDb-p.jDb/1000*1000;
		}
		else
			fehler(7, argv[o[3].i], o[2].name);
	}
	if(o[4].i) {		/* Art der Lagegenauigkeit */
		sscanf(argv[o[4].i], "%i", &gem[p.gSl].lga);
		if(!testLga(gem[p.gSl].lga)) fehler(7, argv[o[4].i], o[4].name);
	}
	/* Jahr der Berechnung prfen */
	if  (p.jDb>900) ibuf = 1000+p.jDb;
	else            ibuf = 2000+p.jDb;
	if(((d.da_year-ibuf)>1)||((d.da_year-ibuf)<0))
		printf("Warnung!\nJahr der Berechnung: %i korrekt?\n\n", ibuf);

	/* Anzeigen aller eingestellten Parameter */
	printf("Eingabedatei: %s\n", inName);
	printf("Ausgabedatei: %s\n", outName);
	printf("Inf-Datei   : %s\n", infName);
	printf("\n");
	printf("Antragsnummer      :          %06li\n", p.aNr);
	printf("zustndige Stelle  : %8s %7li\n", amt[p.zSt].name, amt[p.zSt].nr);
	printf("Gemarkung          : %8s %6li\n", gem[p.gSl].name, gem[p.gSl].nr);
	printf("Jahr der Berechnung:          %4i\n", ibuf);
	printf("Lagegenauigkeit    :          %1i       ", gem[p.gSl].lga);
	if     (gem[p.gSl].lga==1) printf("(AP-Feld)\n");
	else if(gem[p.gSl].lga==3) printf("(PP-Feld)\n");
	printf("\n");


	/* Umsetzen */
	fprintf(outFile, "EDBS03120287AKND000000  0000ULQA00000001%07li       %06li    016WOLF %-6sF                                        8801                                                                                               %-14li             GIAPGSB LK9802  DZENTRALLK9802    3STD172            ASDLBE0005EQLVERMA BBG    \n", amt[p.zSt].nr, p.aNr, argv[1], amt[p.zSt].nr);
	/*                                                        zSt         aNr                                      */
	s=1;
	while(!feof(inFile)) {
		fgets(sbuf, 255, inFile);
		if(feof(inFile)) break;
		nulltoBlank(sbuf,  4,  9);
		nulltoBlank(sbuf, 14, 15);
		nulltoBlank(sbuf, 42, 45);
		nulltoBlank(sbuf, 54, 57);
		sscanf(sbuf,"%i %li %i %i %i %i %li %i %li %i", &p.nb, &p.nr, &p.pa, &p.va, &p.ls, &p.ps, &p.yVk, &p.yNk, &p.xVk, &p.xNk);
		p.ny=p.yVk/100000L; p.nx=p.xVk/100000L;
		p.ps=0;
		if((p.pa==0)||(p.pa==1)) p.lga = 1;
		else if(p.pa==7)         p.lga = 3;
		else                     p.lga = gem[p.gSl].lga + 1;
		fprintf(outFile, "EDBS01850000FEIN%06i  0000ULPUNN  00010001%02i%02i%04i%01i%05li %01i%07li    %03i    %-8s %06li                00000001%03i%07li%03i%07li%03i%1i         %03i         0001000   0000      0                 000000000000\n",s, p.ny, p.nx, p.nb, p.pa, p.nr, p.ps, amt[p.zSt].nr, p.va, gem[p.gSl].name, gem[p.gSl].nr, p.ls, p.yVk, p.yNk, p.xVk, p.xNk, p.lga, p.jDb);
		/*				 "EDBS            s                         ny  nx  nb  pa  nr    ps  zSt      va      gem.name gem.nr                   ls  yVk  yNk xVk  xNk lga         jDb                                                               */
		printf("%5i Punkte bearbeitet\r", s);
		s++;
	}
	fprintf(outFile, "EDBS00240000AEND000000  0000        \n");
	printf("\n");
	return 0;
}
void fehler(int err, ...) {
	va_list ap;
	va_start(ap, err);
	switch(err) {
		case  1: printf("Version 1.02                                  \n"
						"(c) Jrg Schrder                             \n"
						"Aufruf: GSB-EDB <Projekt> <AntrNr> [-optionen]\n"
						"        <Projekt>        Projektname          \n"
						"        <AntrNr>         Antragsnummer        \n"
						"\n"
						"Optionen: -i=<inf-datei> Inf-Datei            \n"
						"          -a=<amt>       Amt-Krzel           \n"
						"                         oder Nummer          \n"
						"          -g=<gemarkung> Gemarkungs-Krzel    \n"
						"                         oder Nummer          \n"
						"          -j=<jahr>      Jahr der Berechnung  \n"
						"          -l=<lga>       Lagegenauigkeit      \n"
						"                         der Festpunkte       \n"
						"                         1 = AP-Feld          \n"
						"                         3 = PP-Feld          \n");
				 break;
		case  2: printf("Kann Datei %s nicht zum Lesen ffnen!\n", va_arg(ap, int));
				 break;
		case  3: printf("Kann Datei %s nicht zum Schreiben ffnen!\n", va_arg(ap, int));
				 break;
		case  4: printf("Umgebungsvariable GSB-EDB nicht gesetzt!\n");
				 printf("Geben Sie mit der Option -i=<infdatei> die INF-Datei an,\n");
				 printf("oder sprechen Sie mit Ihrem Systembetreuer!\n");
				 break;
		case  5: printf("Fehler in INF-Datei\n");
				 printf("Unbekannte Variable in Zeile %d:", va_arg(ap, int));
				 printf(" %s\n", va_arg(ap, int));
				 break;
		case  6: printf("Unbekannte%s: ", va_arg(ap, int));
				 printf("%s\n", va_arg(ap, int));
				 printf("Bitte ergnzen Sie die INF-Datei!\n");
				 break;
		case  7: printf("Ungltiger Wert: %s ", va_arg(ap, int));
				 printf("in Option: %s !\n", va_arg(ap, int));
				 break;
		case  8: printf("Fehler in INF-Datei\n");
				 printf("Unbekannte Lagegenauigkeit in Zeile %i\n", va_arg(ap, int));
				 printf("%s\n", va_arg(ap, int));
				 break;
		case  9: printf("Antragsnummer nicht numerisch: %s\n", va_arg(ap, int));
				 break;
		case 10: printf("Argument %i ist keine Option:", va_arg(ap, int));
				 printf(" %s\n", va_arg(ap, int));
				 break;
		default: printf("Error:%i Undef. Fehler!\n",err);
	}
	va_end(ap);
	exit(err);
	return;
}
void nulltoBlank(char *buf, int a , int e) {
	int i;
	for(i=a; i<e+1; i++) {
		if((buf[i]=='0')||(buf[i]=='.')) buf[i]=' ';
		else break;
	}
	return;
}
char *upstr(char *s) {
	int i = 0;
	while (s[i]) {
		s[i] = toupper(s[i]);
		i++;
	}
	return s;
}
int ssuchSl(struct sl *s, int se, char *st) {
	int i, l;
	l=strlen(st);
	for(i=0; i<se; i++)
		if(!strncmp(s[i].name, st, l)) return i;
	return -1;
}
int lsuchSl(struct sl *s, int se, long nr) {
	int i;
	for(i=0; i<se; i++)
		if(s[i].nr==nr) return i;
	return -1;
}
int testLga(int lga) {
	if((lga==1)||(lga==3)) return 1;
	return 0;
}
int testANr(char *s) {
	int i, l;
	l=strlen(s);
	for(i=0; i<l; i++)
		if(!isdigit(s[i])) return 0;
	return 1;
}